.\" Copyright (c) 1997-2001 Marian Vittek, Bratislava.
.TH XREF 1 \" -*- nroff -*-
.SH NAME
xref \- A cross referencing tool for C and Java.
.SH SYNOPSIS
.hy 0
.na
.TP
.B xref
.RB "[\|" "options"\&\|]
.I infile 
.RB "[\|" "\... infile"\&\|]
.ad b
.hy 1
.SH TYPICAL INVOCATION FORMS
.TP
.B xref
.I -p 
<project>
.TP
.B xref
.I -update -p 
<project>
.TP
.B xref
.I -html -p 
<project>
.TP
.B xref
.I -update -html -p 
<project>
.TP
.TP 
.B xref
<srcdir>
.TP
.B xref 
.I -html
<srcdir>
.SH DESCRIPTION
Xref is a C and Java cross-referencing tool. 
It parses input files noting all symbol's
definitions and usages. If not invoked with the
.I -html
option its output is a list of symbols, together 
with positions of declarations, definition and all usages.
Output is written to the file specified by the 
.I -refs
option. Xref's cross references are destinated to be read by
.I xref
itself or by on-line editing macros.

When invoked with the 
.I -html
option xref  do not produce the cross reference file. Instead it
reads an existing cross reference file and then it
converts input files into a HTML format browsable by standard Internet
browsers. In those files each symbol usage is a hypertext link to the
symbol definition place. If generated with '-htmlgxlist' and '-htmllxlist' 
options then each definition is linked to a
list of all symbol usages. Those cross-references are displayed
in a separate frame of the screen and by clicking on them one can inspect
corresponding source files. 

HTML files are written in a directory hierarchy with the
root specified by the 
.I -htmlroot
option.
.SH NOTE
Xref accepts many options, however except the
.I -p
,
.I -update
and
.I -html
options, it is not recommended to write them directly on the command line. 
The standard way of passing options to xref is the 
.I .xrefrc
file placed in your home directory. Options stored in the ${HOME}/.xrefrc 
file are readed at each xref invocation. Storing command line options into 
this file is the only way how you can pass options to xref when invoked by an
on-line editing macro.

.SH OPTIONS
Here is the table of command options accepted by xref.
.TP
.B \-p project
At the begining of the execution, read and process
options from the section 
.I project
of the 
.I .xrefrc
configuration file. This option is used to abstract from concrete
options and file names when using xref from command line and from several
IDEs.
For more info, see the .xrefrc file documentation.
.TP
.B \-r
This option is default now, so you do not need to enter it. 
It causes that if a directory is entered as 
.I infile
then all files from the directory having the suffix '.c' or '.java'
are processed (note that it is useless to process the '.h' suffixes as
they are processed when included by a '.c' file). 
Subdirectories are recursively searched. This option is obsolete now.
.TP
.B \-I dir
Add the directory
.I dir
to the end of list of
directories inspected when looking for an include file. 
If specified more than one `\|\c
.B \-I\c
\&\|' option,
the directories are scanned in left-to-right order.
.TP
.B -noincluderefresh
This is a particular option causing that xref task do not update
include references when used by an on-line editing macro. This 
option is strongly recommended when working on large projects or on
projects including large header files (for example when including
standard MS-Windows headers).
.TP
.BI "\-D" "name"\c
\&
Predefine \c
.I name\c
\& as a macro, with definition `\|\c
.B 1\c
\&\|'.
.TP
.BI "\-D" "name" = definition
\&
Predefine \c
.I name\c
\& as a macro, with definition \c
.I definition\c
\&.
.TP
.B \-jdkclasspath paths
If you are using JDK from different provider than Sun microsystems, 
you need to specify the classpath (if any) implicitly used by Java jdk tools,
meaning standard .jar archives which are not present in -classpath.
The syntax of paths is the same as for the -classpath option.
For example on usual installation 
the '-jdkclasspath /usr/local/jdk1.2.2/jre/lib/rt.jar' option do the job.
.TP
.B \-classpath paths
This option specifies the classpath used for finding Java .class files.
This option overrides the value of the environment variable CLASSPATH.
See also the '-jdkclasspath' and '-sourcepath' option.
.TP
.B \-sourcepath paths
This option specifies paths where sources of Java packages and classes
are searched. This option overrides the value of the environment 
variable SOURCEPATH.
If not present the current classpath is used as default. The 'sourcepath'
value is used for finding of the source .java file of a class in the
same way as 'classpath' is used to find .class compiled file.
In case when both .java and .class file exists then
times of the last modification of both files are examined and the class is
loaded from the newer file. The 'sourcepath' value is also  used
with the -packages option, when you insert packages names on command
line instead of input file names.
.TP
.B \-packages
This option allows to enter package names instead of input file
names on command line.
.TP
.B \-refs file
This option specifies the file where the cross-references
are stored. If the 
.I -refnum=
option is present then the 
.I -refs 
option indicate the name of directory when the cross-reference
files are stored.
.TP
.B "\-refnum=<number>"
This option specifies the number of files into which the cross 
references are splitted.
When 
.I <number>
== 1, the name specified by the '-refs' option is directly
the name of the cross reference file. 
When 
.I <number>
> 1 then the name specified by the '-refs' option is 
interpreted as a directory name where cross references are stored.
You should specify the 
.I <number>
proportionally to the size
of your project.
This option is meningfull only with combination with the
.I -refs
option. Do not forget to delete the old cross-reference file if
you change the -refnum from 1 to a bigger value.
.TP
.B "\-refalphahash"
Split references into 28 files depending on the first letter of the
symbol. This option is usefull when generating HTML form, because
it makes easy to find cross references for given symbol name.
This option excludes using of the -refnum option. 
.TP
.B "\-refalpha2hash"
Split references into 28*28 files depending on the first two letters of the
symbol. This option is usefull when generating HTML form, because
it makes easy to find cross references for given symbol name.
This option excludes using of the -refnum option. 
.TP
.B "\-update"
Modification time of all input files as well as those listed 
in the existing cross-reference file are checked and only those
having newer modification time than existing cross-references
are scheduled to be processed. Also all files which includes (by
Cpp pre-processor) those files are scheduled to be processed.
This option 
is used for huge projects to maintain the cross-reference file
up to date when only several files were changed.
.TP
.B "\-fastupdate"
The same as -update with the difference that files which include
modified files are not scheduled to be processed. This is
satisfactory for most references to be correctly updated.
.TP
.B "\-fupdate"
Force update. This option is like the 
.I -update
option with the difference that input files entered on command
line are allways parsed (not depending if their modification time
is newer than the cross-reference file or not).
.TP
.B "\-set <name> <value>"
The xref task provide a simple environment management. The -set option
associates the <name> string to the <value> string, the value can be then
retrieved from xref server task by (X)Emacs by using a -get <name> option. 
Usually, the -set option is used to set some project depending
values used by Emacs macros. For the moment strings "compilefile", "compiledir"
and "compileproject" are recognized as <name>. In those cases
the corresponding <value>
fields should be commands invoking the corresponding compiling command (if
such a command contains %s character, it will be respectively
replaced by the corresponding file, directory or project name).
For more info see on-line help for the corresponding Xrefactory
macros in (X)Emacs.
.TP
.B "\-brief"
The output of cross referencing will be in compact form, still readable by
.I xref
but rather difficult for human reader. This is default now.
.TP
.B "\-nobrief"
The output of cross referencing will containing "commentary" records
destinated for human readers.
.TP
.B "\-no_enum"
Don't cross reference enumerators.
.TP
.B "\-no_mac"
Don't cross reference macros.
.TP
.B "\-no_type"
Don't cross-reference user defined types.
.TP
.B "\-no_str"
Don't cross reference structure records.
.TP
.B "\-no_local"
Don't cross reference function arguments and local variables.
.TP
.B "\-no_cfrefs"
Don't get cross references from class files. This can decrease the size of
your Tag file. However, because informations about class hierarchy are taken 
mainly from class files this option may causes that showed class hierarchies
will be incomplete.
.TP
.B \-html
Causes xref to not produce the cross reference file. Instead it reads
a previously generated cross reference file, then it proceeds input files
creating browsable files in the HTML format.
.TP
.B \-htmlroot=dir 
Write output HTML files into the directory hierarchy starting by the
.I dir.
This option is meningfull only with combination with the
.I -html
option.
.TP
.B \-d dir
Equivalent to -htmlroot=dir.
.TP
.B \-htmltab=number
When generating HTML files, set the tabulator to
.I number.
This option is meningfull only with combination with the
.I -html
option.
.TP
.B \-htmllinenums
When converting source files into HTML format generate 
line numbers before source code.
.TP
.B \-htmlnocolors
When converting source files into HTML format do not generate 
color highlighting of keywords, commentaries and preprocessor
directives.
.TP
.B \-htmlgxlist
When converting source files into HTML format generate lists of all 
usages for each 
.I global
symbol. 
.TP
.B \-htmllxlist
When converting source files into HTML format generate lists of all 
usages for each 
.I local
symbol. 
.TP
.B \-htmlfunseparate
Causes that the HTML files will contain a horizontal bar separating function
definitions.
.TP
.B \-htmlcutpath=<path>
This option causes that the output files starting by the path
.I path
are not stored with the full file-path under the -htmlroot directory. 
Rather the listed path is cutted. Xref accepts several '-htmlcutpath' 
options, they are processed in order in which they appear in command line.
On MS-DOS (QNX) system, the path should be an absolute path
but without drive (node) specification.
.TP
.B \-htmlcutcwd
equals '-htmlcutpath=${CWD}' cutting current working directory path
from html paths.
.TP
.B \-htmlcutsourcepaths
cut also all Java source paths defined by CLASSPATH environment variable
or by -sourcepath option.
.TP
.B \-htmllinkcolor=<color>
This option specifies the color in which links of the HTML document will
appear. For example '-htmllinkcolor=black' or '-htmllinkcolor=#000000'
causes that links will be black.
.TP
.B \-htmllinenumcolor=<color>
This option specifies the color of line numbers for generated HTML.
For example '-htmllinenumcolor=black' or '-htmllinenumcolor=#000000'
causes that line numbers will allways be black. This option is meaningfull
only in combination with -htmllinenums option.
.TP
.B \-htmlnounderline
This option causes that links in the HTML document will not be underlined
(if your browser support styles).
.TP
.B \-htmlgenjavadoclinks
When generating html generate links to Javadoc documentation for symbols
without definition reference. Such a link is generated only for
symbols from packages listed in 
the -htmljavadocavailable option. See also -htmljavadocpath option.
.TP
.B \-htmljavadocavailable=<packs>
Specifies for which packages there is a Javadoc documentation available.
It is describing yet existing java documentation and is supposed
to specify packages for which you do not have source code,
but you have a Javadoc generated documentation.
The <packs> string is a list of packages separated by ':' sign. For example
for standard jdk java documentation insert: "-htmljavadocavailable=java.applet:java.awt:java.beans:java.io:java.lang:java.math:java.net:java.rmi:java.security:java.sql:java.text:java.util:javax.accessibility:javax.swing:org.omg.CORBA:org.omg.CosNaming".
See also -htmlgenjavadoclink and -htmljavadocpath options.
.TP
.B \-htmljavadocpath=<path>
Specifies where existing Javadoc documentation is stored. The default
is "-htmljavadocpath=http://java.sun.com/j2se/1.3/docs/api".
It is describing the place where yet existing java documentation is stored
and it is supposed
to specify packages for which you do not have source code,
but you have a Javadoc generated documentation.
See also -htmlgenjavadoclink and -htmljavadocpath options.
.TP
.B \-htmlzip=<command>
This option causes that the command
.I command
is called after having created any HTML file. The character '!' included 
in the 
.I command
has special meaning and is replaced by the name of the last generated
HTML file. For example 
.I """-htmlzip=gzip -f !"""
option causes that all
generated HTML files will be zipped. See also '-htmllinksuffix' option.
.TP
.B \-htmllinksuffix=<suf>
This option causes that whenever a hypertext link to a file xxx should be 
generated then in reality a link on the file xxx.suf is generated. This
option is meaningfull mainly with the '-htmlzip' option. For example
the combination 
.I """-htmlzip=gzip -f !""" -htmllinksuffix=.gz 
causes
that all generated HTML files will be compressed and the links are
correctly generated to those compressed files.
.TP
.B \-stdop file
This option is obsolete, it is kept for backward compatibility reasons, but 
we do not recommend its usage.
This option causes that the file `\|\c
.I file\c
\&\|' is readed and scanned for further options. 
It is supposed that you define option file(s) containing project
specific options and this file will be read by the
.I xref 
at each invocation.
A special mecanism is adopted 
when no option 
.I stdop
is given on the command line. In such a case the project depending 
options are searched in your home directory in the `\|\c
.I .xrefrc
\&\|' file. If this file is not present a file named `\|\c
.I Xref.opt
\&\|' is searched in the directory of the input file. 
If not found it is searched in its super-directory, 
super-super-directory and so on. 
.TP
.B \-no_stdop
This option is obsolete, it is kept for backward compatibility reasons, but 
we do not recommend its usage.
Don't search for the standard option files '.xrefrc' and 'Xref.opt'.
.TP
.B -mf<n>
This option sets the memory factor for cross-references. The larger is the
memory factor, the larger is the memory reserved to store cross-references
and so xref do not need to swap references on disk so frequently. The default
value of 
.I n
is 1. For example the '-mf10' option causes that the memory used for 
cross-references will be increased by the factor of 10. This option can be
used only as command line option. If you use it inside the ~/.xrefrc file
it has no effect.
.TP
.B "\-pass<n>"
Strictly speaking this is not xref option. It is used in the .xrefrc
configuration file to describe how to process source files in multiple passes.
Multiple passes are necessary if you need to process your C sources
for several initial macro settings. For more informations about this option
read the 'xrefrc' manual page.
.TP
.B "\-keep_old"
This option is obsolete. It is kept for compatibility reasons only.
It is recommended to use multiple passes options of the '.xrefrc'
file instead. The '-keep_old' option causes that
the output of cross-referencing is added to 
existing references from the file specified by the 
.I -refs
option. All old references of the reference file are kept.
This option is used when you have 
in one source file different parts, processed 
only when diferent combinations of macros are defined (or undefined) and
you wish that resulting cross-references contain all references from all the
parts of your source file. In such a case you have to proceed several
passes. First processing the first macro setting and then re-pass the cross 
referencer for the same source file with another macro setting and 
with 
the option
.I -keep_old
set. This causes that references from both passes are included in the
resulting cross-reference file
This option is meningfull only with combination with the
.I -refs
option.
.TP
.B "\-no_cpp_comment"
When scanning input files, do not consider the string `\|\c
.I //\c
\&\|' as beggining of a comment ended by end of line.
.TP
.B "\-errors"
Report all error messages.
.TP
.B "-debug"
Produce debugging output, This output facilitate to determine from where
come possible syntax errors.
.SH AUTHOR
Marian Vittek, Bratislava 1997-2001.
.SH "MORE INFO"
.RB http://www.xref-tech.com
.SH "SEE ALSO"
.RB xrefrc(5),
.RB xrefspeller(1)
