.\" Copyright (c) 1999-2001 Marian Vittek, Bratislava.
.TH XREFRC 5 \" -*- nroff -*-
.SH NAME
 .xrefrc \- file containing xref options.
.SH DESCRIPTION

The '.xrefrc' file ('_xrefrc' under MS-DOS) is configuration 
file for the xref cross-referencing
tool. It should be placed in user's home directory from where it is
readed at each xref invocation (under MS-DOS if the %HOME% environement
variable is not set, this file should be placed in the C:\\ root folder).
The file contains xref command
line switches used for processing your projects, 
mainly information about source files, 
include directories ( '-I <dir>' 
option )
and  default  macro-definitions ( '-D<name>' option). 
It should equally  contain  the  '-refs  <file>'
option  specifying  where  the
resulting file with cross references should be searched or created. 
.I Informations about available options can be found in the 'xref' 
.I manual page ('xref.man' file). 
If your sources or compiler  use  non-standard  include  directories  you
should  also  place this information  here.  The '.xrefrc' file can contain 
comments starting by two slashes
followed by at least one blank character and ending by end of the line.

In  order  to  handle multiple projects   the  '.xrefrc'
file  is  composed  from  separate
sections. Each section starts with a  section  marker  stating  the
project  name.  Options defined inside the section are taken into account
if 'xref' is invoked with the '-p'
option followed by the name of this section. 
In order to pass '-p' option to xref from Emacs/XEmacs
editors, you have to set current project name with
the 'M-x xref-project-set-active' function or by selecting it throw
the 'Xref->Project->Set Active" menu item.

Lets take for example the '.xrefrc' file defining two projects
('solver' and 'player'):

---------------------------------------------------------

[solver]        // first section
.br
-r /home/marian/solver 
.br
-r /home/marian/commons
.br
-I /usr/lib/gcc-lib/i386-redhat-linux/2.7.2.3/include
.br
-DTARGET=linux
.br
-refs /home/marian/XSrefs
.br
-htmlroot=/home/marian/HTML
.br

[player]          // second section
.br
-r /home/marian/player
.br
-classpath /usr/share/jdk/classes.zip:/java
.br
-refs /home/marian/Xplay
.br
-htmlroot=/home/marian/HTML

---------------------------------------------------------

Invocation via the 'xref -p solver' command will cause that
(recursively) all source files from the '/home/marian/solver' 
and '/home/marian/commons' directories will be cross-referenced.
Also the 'TARGET' macro will be defined having the value 'linux'. The 
resulting cross-references will be stored into the '/home/marian/XSrefs'
file. 

An invocation with the 'xref -p player' command will cause
that source files from the '/home/marian/player' directory will 
be processed with the classpath set to '/usr/share/jdk/classes.zip:/java'
and the resulting
cross-references will be stored in the '/home/marian/Xplay'
file.

An invocation with the 'xref -html -p player'
command causes that all files from the '/home/marian/player' 
directory will be processed
with the classpath set to '/usr/share/jdk/classes.zip:/java',
the input files will be
converted to the HTML format and the resulting HTML files
will be stored in a file hierarchy under the '/home/marian/HTML'
directory.

If you  are using Xref from Emacs/Xemacs then all interactive
requests will use options from the section entered to the
last 'M-x xref-project-set-active' invocation.

NOTE: If the '.xrefrc' file contains input file names, they are
processed only when invoked with '-p' or '-create' options on
the command line.

.I AUTOMATIC PROJECT DETECTION

Xrefactory contains some support for automatic project detection.
If under Emacs you select "automatic project detection" then 
Xrefactory tries to detect active project automatically infering
it from available project names and from the path of currently edited
file.

In practice it means that project names are in fact names of directories
containing source files. A project is detected as active if
the project name is a directory containing (directly or in several
subdirectory levels) the currently edited file.
For example, let's take the .xrefrc file:

---------------------------------------------------------

[/home/marian/solver]        // first section
.br
-I /usr/lib/gcc-lib/i386-redhat-linux/2.7.2.3/include
.br
-I /home/marian/include
.br
-I /mygroup/include
.br
-DTARGET=linux
.br
-refs /home/marian/XSrefs


[/home/marian/xref]          // second section
.br
-I /usr/lib/gcc-lib/i386-redhat-linux/2.7.2.3/include
.br
-I /home/marian/include
.br
-I /mygroup/include
.br
-DNODEBUG
.br
-refs /home/marian/XXrefs

---------------------------------------------------------

The first part of the file is triggered for all source files
with full path name starting by
/home/marian/solver
directory, the second part is triggered for  all
files under '/home/marian/xref' directory. For example if xref is invoked for
a  file,  saying  
/home/marian/xref/src/main.c
this  file  will  be
processed with options read from the second part (meaning among others
that   cross   references   will   be   searched/stored  in  the  file
/home/marian/XXrefs
). However, 'xref' is unable to switch between different sections 
in one single run, meaning if you type the 
command 'xref -r /home/marian' then all files will be
processed with options from the same section of the '.xrefrc' file,
the selected section depends on the first processed file.

NOTE:  It  is  strongly  recommended  to use all file names with their
absolute paths.



.SH MULTIPLE PRE-PROCESSOR PASSES


In the C language the same sources are often compiled multiple
times with only different initial macro settings. Usually the initial
macro setting is used to remove or to insert a part 
of source code enclosed in the pre-processor 
directives #if-#else-#endif. In such a case the '#if part' of code
is never compiled together with the '#else part' of the code. However,
for the purpose of cross referencing and (especially) for manipulations
like renaming, it is essential to have together the cross-references
from both parts of the '#if-#else-#endif' construction. For this reason
xref permits you to define multiple passes throw the source code
collecting all references from those passes. Usually you will use
one pass to one possible compilation of sources (for example for one possible
platform). Multiple
passes are specified at the '.xrefrc' file level by using special
options '-pass<n>', where <n> is the pass number. If such an option
occurs it causes that all following options (except other '-pass'
option) are taken into account only when making the <n>-th pass throw
source code. By default xref makes as many passes throw source code
as is the greatest <n> used in the '-pass' options. For example:


---------------------------------------------------------

[/home/marian/solve]
.br
-r /home/marian/solve
.br
-refs /home/marian/XSrefs
.br
-pass1
.br
-DTARGET=linux
.br
-pass2
.br
-DTARGET=solaris

---------------------------------------------------------

This .xrefrc file causes that all source files
from the '/home/marian/solve' directory will be processed twice
first time with options:

.br
-r /home/marian/solve
.br
-refs /home/marian/XSrefs
.br
-DTARGET=linux
.br

and second time with options:

.br
-r /home/marian/solve
.br
-refs /home/marian/XSrefs
.br
-DTARGET=solaris

References from both passes will be colected and inserted into
the '/home/marian/XSrefs' file.

.I IMPORTANT NOTE: 
The multiple passes throw sources will increase
the time of file processing and especially they will
considerably slowdown several on-line editing actions, 
to be used with moderation!

.SH AUTHOR
Marian Vittek, Bratislava 1997-2001.
.SH "MORE INFO"
.RB http://www.xref-tech.com
.SH "SEE ALSO"
.RB xref(1),
.RB xrefspeller(1)



